@@ -60,6 +60,8 @@
      "MOZMOZEAL:%s:%d: " FMT "\n", __FUNCTION__, __LINE__, ## ARG)
 #define TRACE_LOG(FMT, ARG...) pr_mozeal_trace(\
      "MOZEAL_TRACE:%s:%s:%d: " FMT "\n", __FILE__, __FUNCTION__, __LINE__, ## ARG)
+void
+pr_mozeal_trace (const char *fmt, ...);
 #else
 #       define ULOG_DEBUG_F(...) do {} while (0)
 #       define ULOG_CRIT_F(...) do {} while (0)
@@ -175,6 +177,7 @@
 #define G_MOZILLA_PREF_BUFFER_DEF_WIDTH                 "microb.buffer.height"
 #define G_MOZILLA_PREF_BUFFER_DEF_HEIGHT                "microb.buffer.width"
 #define G_MOZILLA_PREF_HIDE_FIXED_FRAMES                "microb.hide_fixed_frames"
+#define G_MOZILLA_PREF_LAYOUT_CSS_DPI                   "layout.css.dpi"
 
 #define G_MOZILLA_LOCAL_HOST_IP                         "127.0.0.1"
 #define G_MOZILLA_TRANSLATE_CONNECTIVITY_ERROR          "Network Connection Error"
