@@ -68,7 +68,7 @@ MicrobEalNotification::AddButtons(nsIArray *aButtons)
 }
 
 nsresult
-MicrobEalNotification::AddToUi(nsString &aTitle)
+MicrobEalNotification::AddToUi(const nsString &aTitle)
 {
   GMozillaEngine *engine = NULL;
   GetEngine(&engine);
@@ -80,6 +80,10 @@ MicrobEalNotification::AddToUi(nsString &aTitle)
   mNotificationWidget = g_mozilla_append_notification(engine, utf8Label, this);
   g_object_ref(mNotificationWidget);
 
+  // Store title for this notification
+  if (mTitle.IsEmpty() && !aTitle.IsEmpty())
+    mTitle = aTitle;
+
   const PRUint32 arraySize=mButtons.Count();
   for (PRUint32 i=0; i<arraySize; i++) {
     MicrobEalNotificationButton *button = mButtons[i];
@@ -90,7 +94,7 @@ MicrobEalNotification::AddToUi(nsString &aTitle)
     NS_Free(buttonUtf8Label);
   }
 
-  g_mozilla_show_notification(engine, mNotificationWidget, this, aTitle.IsEmpty() ? nsnull : NS_ConvertUTF16toUTF8(aTitle).get());
+  g_mozilla_show_notification(engine, mNotificationWidget, this, mTitle.IsEmpty() ? nsnull : NS_ConvertUTF16toUTF8(mTitle).get());
 
   NS_Free(utf8Label);
   return NS_OK;
@@ -157,6 +161,9 @@ MicrobEalNotification::GetLabel(nsAString& aLabel)
 NS_IMETHODIMP
 MicrobEalNotification::SetLabel(const nsAString& aLabel)
 {
+  if (CheckLogicalId(aLabel, mLabel))
+    return NS_OK;
+
   mLabel = aLabel;
   return NS_OK;
 }
