@@ -27,9 +27,6 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-#undef __FILE__
-#define __FILE__ "MicrobEalXshmRenderer"
-
 #include "MicrobEalXshmRenderer.h"
 
 #include "common.h"
@@ -215,13 +212,12 @@ nsresult
 MicrobEalXshmRenderer::DrawRectToXshm(nsIDOMWindow *aWindow,
                                       gfxRect &aRect,
                                       MicrobShmBufData &aData,
-                                      int aReqestId,
+                                      int aRequestId,
                                       float aZoomValue)
 {
     nsresult rv;
 
     GMozillaEngine* engine = GetGMozEngineForDOMWindow(aWindow);
-
     NS_ENSURE_TRUE(engine, NS_ERROR_NULL_POINTER);
 
     if (aData.IsBlocked()) {
@@ -242,7 +238,8 @@ MicrobEalXshmRenderer::DrawRectToXshm(nsIDOMWindow *aWindow,
     float innerSW = 0, innerSH = 0;
 
     // Get Inner dimensions to limit requested area
-    rv = MicrobEalContextMenuInfo::GetDomWindowScrollInfo(aWindow, &innerWidth, &innerHeight, &scrollMaxX, &scrollMaxY, &scrollX, &scrollY);
+    PRInt32 bidiOffset = 0;
+    rv = MicrobEalContextMenuInfo::GetDomWindowScrollInfo(aWindow, &innerWidth, &innerHeight, &scrollMaxX, &scrollMaxY, &scrollX, &scrollY, &bidiOffset);
 
     // Send scroll position not equals to -1 only if it was changed by engine
     PRInt32 ux = 0, uy = 0, sx = 0, sy = 0;
@@ -276,20 +273,23 @@ MicrobEalXshmRenderer::DrawRectToXshm(nsIDOMWindow *aWindow,
     if (dRect.YMost() > innerSH)
       dRect.size.height = innerSH - dRect.pos.y;
 
+    float maxZoomL = ceilf(dRect.pos.x + dRect.size.width);
+    float maxZoomT = ceilf(dRect.pos.y + dRect.size.height);
+    
+    float zooml = floorf(dRect.pos.x);
+    float zoomt = floorf(dRect.pos.y);
+    float zoomWidth = ceilf(maxZoomL - zooml);
+    float zoomHeight = ceilf(maxZoomT - zoomt);
+
     // Get result dimensions after zooming
-    int l = (int)roundf(dRect.pos.x * (float)nsIDeviceContext::AppUnitsPerCSSPixel() / aZoomValue);
-    int t = (int)roundf(dRect.pos.y * (float)nsIDeviceContext::AppUnitsPerCSSPixel() / aZoomValue);
-    int width = (int)roundf(dRect.size.width * (float)nsIDeviceContext::AppUnitsPerCSSPixel() / aZoomValue);
-    int height = (int)roundf(dRect.size.height * (float)nsIDeviceContext::AppUnitsPerCSSPixel() / aZoomValue);
+    int l = (int)roundf((zooml - bidiOffset) * (float)nsIDeviceContext::AppUnitsPerCSSPixel() / aZoomValue);
+    int t = (int)roundf(zoomt * (float)nsIDeviceContext::AppUnitsPerCSSPixel() / aZoomValue);
+    int width = (int)roundf(zoomWidth * (float)nsIDeviceContext::AppUnitsPerCSSPixel() / aZoomValue);
+    int height = (int)roundf(zoomHeight * (float)nsIDeviceContext::AppUnitsPerCSSPixel() / aZoomValue);
 
-    float zooml = roundf(dRect.pos.x);
-    float zoomt = roundf(dRect.pos.y);
-    float zoomWidth = roundf(dRect.size.width);
-    float zoomHeight = roundf(dRect.size.height);
-
-    ULOG("or[%g,%g,%g,%g] rt[%i,%i,%i,%i], zrt[%g,%g,%g,%g], aReqestId:%i, inner[%i,%i], scrMax[%i,%i], scr[%i,%i] z:%g\n",
+    ULOG("or[%g,%g,%g,%g] rt[%i,%i,%i,%i], zrt[%g,%g,%g,%g], aRequestId:%i, inner[%i,%i], scrMax[%i,%i], scr[%i,%i] z:%g\n",
          dRect.pos.x, dRect.pos.y, dRect.size.width, dRect.size.height,
-         l, t, width, height, zooml, zoomt, zoomWidth, zoomHeight, aReqestId,
+         l, t, width, height, zooml, zoomt, zoomWidth, zoomHeight, aRequestId,
          innerWidth, innerHeight, scrollMaxX, scrollMaxY, scrollX, scrollY, aZoomValue);
 
     NS_ENSURE_TRUE(aData.data && aData.data != (void*)-1, NS_ERROR_FAILURE);
@@ -298,7 +298,9 @@ MicrobEalXshmRenderer::DrawRectToXshm(nsIDOMWindow *aWindow,
     gfxASurface::gfxImageFormat imageFormat = gfxASurface::ImageFormatRGB24;
 
     // Adjust Image Format
+#ifdef MICROB_API_16BPP_SUPPORT
     if (aData.depth == 16) imageFormat = gfxASurface::ImageFormatRGB16;
+#endif
     if (aData.depth == 32) imageFormat = gfxASurface::ImageFormatARGB32;
     nsRefPtr<gfxImageSurface> surface =
         new gfxImageSurface((unsigned char*)aData.data, gfxIntSize(aData.width, aData.height),
@@ -327,8 +329,10 @@ MicrobEalXshmRenderer::DrawRectToXshm(nsIDOMWindow *aWindow,
     ctx->Scale(aZoomValue, aZoomValue);
 
     PRUint32 renderDocFlags = nsIPresShell::RENDER_IGNORE_VIEWPORT_SCROLLING | nsIPresShell::RENDER_CARET;
+#ifdef MICROB_API_RENDER_NO_FIXED_FRAMES
     if (!engine->mPaintFixedFrames)
       renderDocFlags |= nsIPresShell::RENDER_NO_FIXED_FRAMES;
+#endif
 
     ULOG("Block painting:%p:%i->%i", engine, aData.IsBlocked(), TRUE);
     aData.SetBlocked(true);
@@ -336,12 +340,11 @@ MicrobEalXshmRenderer::DrawRectToXshm(nsIDOMWindow *aWindow,
     presShell->RenderDocument(nr, renderDocFlags, NS_RGB(255, 255, 255), ctx);
 
     ULOG("[%i,%i,%i,%i]:, nr[%i,%i,%i,%i]\n", (int)l, (int)t, (int)width, (int)height, nr.x, nr.y, nr.width, nr.height);
-
     PaintNotifyMessage msg =
-        { (guint32) engine, aReqestId, (int)roundf(zooml), (int)roundf(zoomt),
+        { (guint32) engine, aRequestId, (int)roundf(zooml), (int)roundf(zoomt),
           (int)roundf(zoomWidth), (int)roundf(zoomHeight), scrollMaxX+innerWidth,
           scrollMaxY+innerHeight, scrollX, scrollY, (int)roundf(aZoomValue*100) };
-    ULOG("id:%i, r[%i,%i,%i,%i], z:%i, scr[%i,%i], retval:%i, pff:%i", aReqestId, (int)roundf(zooml), (int)roundf(zoomt), (int)roundf(zoomWidth), (int)roundf(zoomHeight), (int)roundf(aZoomValue*100), scrollX, scrollY, engine->mPaintFixedFrames);
+    ULOG("id:%i, r[%i,%i,%i,%i], z:%i, scr[%i,%i], pff:%i", aRequestId, (int)roundf(zooml), (int)roundf(zoomt), (int)roundf(zoomWidth), (int)roundf(zoomHeight), (int)roundf(aZoomValue*100), scrollX, scrollY, engine->mPaintFixedFrames);
     g_web_bus_send_message(web->web_bus, web->ui_bus_name, PAINT_NOTIFY_MSG, &msg, sizeof(msg));
 
     // Check If Engine Is Active
@@ -385,7 +388,7 @@ MicrobEalXshmRenderer::UpdateUIShmInfo(void)
   NS_ENSURE_TRUE(web, NS_ERROR_FAILURE);
   NS_ENSURE_TRUE(web->web_bus, NS_ERROR_FAILURE);
   ShmInfoMessage msg = { 0, mShmData.id, mShmData.width, mShmData.height, mShmData.depth };
-  g_web_bus_send_message(web->web_bus, G_MOZILLA_UI, SHM_INFO_MSG, &msg, sizeof(msg));
+  g_web_bus_send_message(web->web_bus, web->ui_bus_name, SHM_INFO_MSG, &msg, sizeof(msg));
   return NS_OK;
 }
 
@@ -430,6 +433,11 @@ MicrobEalXshmRenderer::CreateSharedImageData(int width, int height, int depth, G
     // Adjust Depth According To Engine GFX Depth
     depth = atoi(gfxDepth);
 
+#ifndef MICROB_API_16BPP_SUPPORT
+  if (depth == 16)
+    depth = 24;
+#endif
+
   ULOG("XSHMB depth: %i bpp", depth);
 
   if (mShmData.id != -1 && width <= mShmData.width && height <= mShmData.height && depth == mShmData.depth) {
@@ -467,9 +475,9 @@ MicrobEalXshmRenderer::CreateSharedImageData(int width, int height, int depth, G
   // Attach To Current Process
   int16_t *blocked = (int16_t*)shmat(shmId, 0, 0);
   int16_t *data = blocked;
-  data += sizeof(int16_t);
+  data++;
 
-  NS_ENSURE_TRUE(data, NS_ERROR_FAILURE);
+  NS_ENSURE_TRUE(blocked != (int16_t*) -1, NS_ERROR_FAILURE);
 
   shmctl(shmId, IPC_RMID, 0);
 
@@ -513,7 +521,7 @@ MicrobEalXshmRenderer::~MicrobEalXshmRenderer()
 {
   // All windows has been closed, need to reset gweb paint listener pointer
   if (mGWeb && G_IS_OBJECT(mGWeb))
-    g_object_set_data(G_OBJECT(mGWeb), G_MOZILLA_PAINT_LISTENER, nsnull);
+    g_object_steal_data(G_OBJECT(mGWeb), G_MOZILLA_PAINT_LISTENER);
 
   // Disable SHM Buffer Shrink Timer
   EAL_IF_GFREE_FUNC(mShmShrinkTimerId, g_source_remove);
@@ -728,6 +736,8 @@ ExpandByDivElement(nsIDOMNode *aNode, nsIDOMWindow *aWindow, nsSize &cl, nsSize
     nsString styleProp(NS_LITERAL_STRING("style"));
     nsString styleHidden(NS_LITERAL_STRING("overflow: hidden;"));
     nsString styleAppend(NS_LITERAL_STRING("overflow: auto;"));
+    nsString styleHeight(NS_LITERAL_STRING("height:"));
+    nsString styleWidth(NS_LITERAL_STRING("width:"));
     element->GetAttribute(styleProp, style);
 
     PRInt32 pos = style.RFind(styleHidden);
@@ -739,6 +749,14 @@ ExpandByDivElement(nsIDOMNode *aNode, nsIDOMWindow *aWindow, nsSize &cl, nsSize
       style.Append(styleAppend);
       element->SetAttribute(styleProp, style);
     }
+
+    // If height or width is explicitly specified, dont expand in that direction
+    if (kNotFound != style.RFind(styleHeight))
+      hdiff = 0;
+    if (kNotFound != style.RFind(styleWidth))
+      wdiff = 0;
+    if (wdiff <= 0 && hdiff <= 0)
+      return NS_OK;
   }
 
   nsCOMPtr<nsIDOMWindowInternal> mainWinInt = do_QueryInterface(topWindow, &rv);
@@ -863,13 +881,12 @@ ExpandByFrameElement(nsIDOMElement *aNode, nsIDOMWindow *aWindow, nsSize &cl, ns
 static PRBool
 LimitRectBySizeAroundRect(gfxRect &aRect, const gfxSize &aSize, const gfxRect &aArrRect)
 {
-  int vs_w_diff = aRect.size.width - aSize.width;
-  int vs_h_diff = aRect.size.height - aSize.height;
-  if (vs_w_diff > 0)
-    aRect.Inset(vs_w_diff);
-  if (vs_h_diff > 0)
-    aRect.Inset(vs_h_diff);
-  if (vs_w_diff > 0 || vs_h_diff > 0) {
+  gfxFloat vs_w_diff = (aRect.size.width - aSize.width) / 2;
+  gfxFloat vs_h_diff = (aRect.size.height - aSize.height) / 2;
+  vs_w_diff = (vs_w_diff > 0) ? vs_w_diff : 0;
+  vs_h_diff = (vs_h_diff > 0) ? vs_h_diff : 0;
+  aRect.Inset(vs_h_diff, vs_w_diff, vs_h_diff, vs_w_diff);
+  if (vs_w_diff || vs_h_diff) {
     aRect = aRect.Union(aArrRect);
     return PR_TRUE;
   }
@@ -980,6 +997,12 @@ NS_IMETHODIMP MicrobEalXshmRenderer::HandleEvent(nsIDOMEvent *aEvent)
   GMozillaEngine* engine = GetGMozEngineForDOMWindow(eventWin);
   NS_ENSURE_TRUE(engine, NS_OK);
 
+  if (engine->recalculate_element)
+  {
+    recalculate_element_info(engine, TRUE);
+    engine->recalculate_element = FALSE;
+  }
+
   DisableScrollbars(eventWin);
 
   nsCOMPtr <nsIDOMWindowInternal> intWin;
@@ -1002,7 +1025,6 @@ NS_IMETHODIMP MicrobEalXshmRenderer::HandleEvent(nsIDOMEvent *aEvent)
   gfxRect target_rect(engine->mLastTargetArea.x, engine->mLastTargetArea.y,
                       engine->mLastTargetArea.width, engine->mLastTargetArea.height);
   gfxRect resultRect(0,0,0,0);
-  gfxRect intr(0,0,0,0);
   gfxRect intr_target(0,0,0,0);
 
   gfxRect visrectPrev(visrect);
@@ -1015,10 +1037,6 @@ NS_IMETHODIMP MicrobEalXshmRenderer::HandleEvent(nsIDOMEvent *aEvent)
 
   LimitRectBySizeAroundRect(target_rect, gfxSize(SHM_MAX_WIDTH, SHM_MAX_HEIGHT), visrect);
 
-  PRInt32 ux=0,uy=0,sx=0,sy=0;
-  ScrollChanged(engine, scrollX, scrollY, ux, uy, sx, sy);
-  PRBool scrollDidChange = (ux || uy);
-
   float l = 0, t = 0, w = 0, h = 0;
   for (PRUint32 kr = 0; kr < aLength; kr++) {
     rv = clientRects->Item(kr, getter_AddRefs(rect));
@@ -1029,9 +1047,7 @@ NS_IMETHODIMP MicrobEalXshmRenderer::HandleEvent(nsIDOMEvent *aEvent)
       rect->GetHeight(&h);
       rc.pos.x = l + scrollX; rc.pos.y = t + scrollY;
       rc.size.width = w; rc.size.height = h;
-      ULOG("Rect[%i]:[%g,%g,%g,%g]", kr, rc.pos.x, rc.pos.y, rc.size.width, rc.size.height);
-      intr = scrollDidChange ? rc : rc.Intersect(visrect);
-      if (!intr.IsEmpty()) {
+      if (!rc.IsEmpty()) {
         intr_target = rc.Intersect(target_rect);
         if (!intr_target.IsEmpty()) {
           resultRect = resultRect.Union(intr_target);
@@ -1039,6 +1055,11 @@ NS_IMETHODIMP MicrobEalXshmRenderer::HandleEvent(nsIDOMEvent *aEvent)
       }
     }
   }
+
+  if (engine->full_update_required) {
+    engine->full_update_required = FALSE;
+    resultRect = resultRect.Union(gfxRect(0, 0, engine->mLastVisibleArea.width, engine->mLastVisibleArea.height));
+  }
   ULOG("ResRect:[%g,%g,%g,%g], scr[%i,%i]", resultRect.pos.x, resultRect.pos.y, resultRect.size.width, resultRect.size.height, scrollX, scrollY);
 
   intr_target.pos.x = resultRect.pos.x * engine->mZoomValue;
@@ -1049,6 +1070,11 @@ NS_IMETHODIMP MicrobEalXshmRenderer::HandleEvent(nsIDOMEvent *aEvent)
   if ((mShmData.IsBlocked() || !engine->is_active) && !resultRect.IsEmpty()) {
     if (!engine->mBlockedUpdates)
       engine->mBlockedUpdates = gdk_region_new();
+
+    gfxRect tr(engine->mLastTargetArea.x * engine->mZoomValue, engine->mLastTargetArea.y * engine->mZoomValue,
+               engine->mLastTargetArea.width * engine->mZoomValue, engine->mLastTargetArea.height * engine->mZoomValue);
+    intr_target = intr_target.Intersect(tr);
+
     GdkRectangle r = { intr_target.pos.x, intr_target.pos.y, intr_target.size.width, intr_target.size.height };
     gdk_region_union_with_rect(engine->mBlockedUpdates, &r);
     ULOG("Painting is blocked:%p:%i, r[%i,%i,%i,%i]", engine, mShmData.IsBlocked(), r.x, r.y, r.width, r.height);
@@ -1072,6 +1098,10 @@ NS_IMETHODIMP MicrobEalXshmRenderer::HandleEvent(nsIDOMEvent *aEvent)
     if (NS_FAILED(CreateSharedImageData(intr_target.size.width, intr_target.size.height, SHM_DEPTH)))
       return NS_OK;
 
+    gfxRect tr(engine->mLastTargetArea.x * engine->mZoomValue, engine->mLastTargetArea.y * engine->mZoomValue,
+               engine->mLastTargetArea.width * engine->mZoomValue, engine->mLastTargetArea.height * engine->mZoomValue);
+    intr_target = intr_target.Intersect(tr);
+
     // Draw Rect To SHMB
     MicrobEalXshmRenderer::DrawRectToXshm(eventWin, intr_target, mShmData, -1, engine->mZoomValue);
   }
